@@ -1,7 +1,7 @@
 /*
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 ART_454 -- Artificial Read Transcriber 
-Copyright(c) 2008-2011 Weichun Huang, All Rights Reserved.
+Copyright(c) 2008-2012 Weichun Huang, All Rights Reserved.
 _____________________________________________________________________________________________________________
 */
 #include <iostream>
@@ -22,13 +22,17 @@ using namespace std;
 
 int main(int argc, char* argv[]){
     cout <<"==================================================================="<<endl;
-    cout <<"                      ART_454 (Version 1.6.8)                      "<<endl; 
+    cout <<"                      ART_454 (Version 2.1.8)                      "<<endl; 
     cout <<"                   Simulation of 454 Pyrosequencing                "<<endl; 
-    cout <<"     Copyright (c) 2008-2011, Weichun Huang. All Rights Reserved.  "<<endl; 
+    cout <<"     Copyright (c) 2008-2012, Weichun Huang. All Rights Reserved.  "<<endl; 
     cout <<"==================================================================="<<endl<<endl;
 
     bool sam_out=false;
+    bool aln_out=false;
+    bool titanium=false;
+    bool user_cycle=false;
     string profile_name = "";
+    int num_flow_cycles = 100;
     int i=1;
     for(;i<argc;++i){
 	    char* pch = argv[i];
@@ -39,10 +43,23 @@ int main(int argc, char* argv[]){
 			    case 'P':
 				    if(i<argc) profile_name=argv[++i]; 
 				    break;
+			    case 'a':
+			    case 'A':
+				    aln_out=true;
+				    break;
 			    case 's':
 			    case 'S':
 				    sam_out=true;
 				    break;
+			    case 't':
+			    case 'T':
+				    titanium=true;
+				    break;
+			    case 'c':
+			    case 'C':
+				    if(i<argc) num_flow_cycles = atoi(argv[++i]);
+				    user_cycle=true;
+				    break;
 			    default:
 				   cerr<<"Error: unreconized option \""<<*pch<< "\""<<endl;
 				   break;
@@ -50,19 +67,26 @@ int main(int argc, char* argv[]){
 	    }
     }
 
+    if(!user_cycle && titanium){
+	   num_flow_cycles=200;
+    }
+
     int k=argc-i+1;
     if(k !=4 &&  k !=6){
         cout<<"USAGE FOR SINGLE-END READ SIMULATION"<<endl<<endl;
-        cout<<"     "<< PRGNAME <<" [-s] [ -p read_profile ] <INPUT_SEQ_FILE> <OUTPUT_FILE_PREFIX> <FOLD_COVERAGE>"<<endl<<endl;
+        cout<<"     "<< PRGNAME <<" [-a ] [-s] [-t] [ -p read_profile ] [ -c num_flow_cycles ] <INPUT_SEQ_FILE> <OUTPUT_FILE_PREFIX> <FOLD_COVERAGE>"<<endl<<endl;
        	cout<<"     Example:"<<endl;
        	cout<<"             "<<PRGNAME <<" seq_reference.fa -s ./outdir/dat_single_end 20"<<endl<<endl;
         cout<<"USAGE FOR PAIRED-END READ SIMULATION"<<endl<<endl;
-        cout<<"     "<< PRGNAME <<" [-s] [ -p read_profile ] <INPUT_SEQ_FILE> <OUTPUT_FILE_PREFIX> <FOLD_COVERAGE> <MEAN_FRAG_LEN> <STD_DEV>"<<endl<<endl;
+        cout<<"     "<< PRGNAME <<" [-a ] [-s] [-t] [ -p read_profile ] [ -c num_flow_cycles ] <INPUT_SEQ_FILE> <OUTPUT_FILE_PREFIX> <FOLD_COVERAGE> <MEAN_FRAG_LEN> <STD_DEV>"<<endl<<endl;
         cout<<"     Example:"<<endl;
        	cout<<"             "<<PRGNAME <<" seq_reference.fa -s ./outdir/dat_paired_end 20 500 20"<<endl<<endl;
         cout<<"------------------------Optional Parameters------------------------"<<endl<<endl;
+        cout<<"-a indicate to output the ALN alignment file"<<endl<<endl;
+        cout<<"-s indicate to output the SAM alignment file"<<endl<<endl;
+        cout<<"-t indicate to simulate reads from the built-in GS FLX Titanium profile [default: GS FLX profile]"<<endl<<endl;
+        cout<<"-c specify the number of flow cycles by the sequencer [ default: 100 for GS-FLX, and 200 for GS-FLX Titanium ] "<<endl<<endl;
         cout<<"-p specify user's own read profile for simulation"<<endl<<endl;
-        cout<<"-s indicate to generate a SAM alignment file"<<endl<<endl;
        	cout<<"     Note: the name of a read profile is the directory containing read profile data files."<<endl;
         cout<<"           please read the REAME file about the format of 454 read profile data files and."<<endl;
         cout<<"           and the default filenames of these data files." <<endl<<endl; 
@@ -99,9 +123,12 @@ int main(int argc, char* argv[]){
     ofstream FQFILE(fqfile.c_str(),ios::binary);
     if(!FQFILE.is_open()) { cout<<"can not open output file: "<<fqfile<<endl; exit(0); }
 
-    ofstream ALNFILE(alnfasta.c_str(),ios::binary);
-    if(!ALNFILE.is_open()) { cout<<"can not open output file: "<<alnfasta<<endl; exit(0); }
-    ALNFILE<<"##ART_454"<<endl;
+    ofstream ALNFILE;
+    if(aln_out) {
+	    ALNFILE.open(alnfasta.c_str(),ios::binary);
+	    if(!ALNFILE.is_open()) { cout<<"can not open output file: "<<alnfasta<<endl; exit(0); }
+	    ALNFILE<<"##ART_454"<<endl;
+    }
 
     ofstream STATFILE(statfile.c_str(),ios::binary);
     if(!STATFILE.is_open()) { cout<<"can not open output file: "<<statfile<<endl; exit(0); }
@@ -109,7 +136,7 @@ int main(int argc, char* argv[]){
     read_profile::set_err_prob();
     read_profile qdist; 
     if(profile_name.empty()){
-	    qdist.default_profile(); 
+	    qdist.default_profile(titanium); 
     }
     else{
 	    qdist.user_profile(profile_name+"/qual_1st_profile", profile_name+"/qual_mc_profile", profile_name+"/indel_error_profile", profile_name+"/length_dist");
@@ -125,7 +152,9 @@ int main(int argc, char* argv[]){
     sH.ID="02";
     sH.PN="ART_454"; 
     for(int i=0;i < argc; ++i) { sH.CL.append(argv[i]);sH.CL.append(" "); }
-    sH.printAlnHeader(ALNFILE);
+    if(aln_out){
+	    sH.printAlnHeader(ALNFILE);
+    }
 
     if(sam_out){
 	    sH.printHeader(SAMFILE);
@@ -159,10 +188,13 @@ int main(int argc, char* argv[]){
         ofstream FQFILE2(fqfile2.c_str(),ios::binary);
         if(!FQFILE2.is_open()) { cout<<"can not open output file: "<<fqfile2<<endl; exit(0); }
 
-        ofstream ALNFILE2(alnfasta2.c_str(),ios::binary);
-        if(!ALNFILE2.is_open()) { cout<<"can not open output file: "<<alnfasta2<<endl; exit(0); }
-       	ALNFILE2<<"##ART_454"<<endl;
-       	sH.printAlnHeader(ALNFILE2);
+	ofstream ALNFILE2;
+	if(aln_out) {
+	       	ALNFILE2.open(alnfasta2.c_str(),ios::binary);
+	       	if(!ALNFILE2.is_open()) { cout<<"can not open output file: "<<alnfasta2<<endl; exit(0); }
+	       	ALNFILE2<<"##ART_454"<<endl;
+	       	sH.printAlnHeader(ALNFILE2);
+       	}
 
         seqRead a_read_2;
         vector<short> qual_2;
@@ -209,12 +241,12 @@ int main(int argc, char* argv[]){
 		short start_cyc=0; //reset start_cyc to zero 
 		bool ok=true;
                 if(a_read.is_plus_strand){ 
-                  qdist.get_read_qual_fast(a_art.homo_plus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual, start_cyc);
-                  ok=qdist.get_read_qual_fast(a_art.homo_plus, a_read_2.bpos, a_read_2.seq_ref, a_read_2.seq_read, aln2, qual_2, start_cyc);
+                  qdist.get_read_qual_fast(a_art.homo_plus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual, start_cyc, num_flow_cycles);
+                  ok=qdist.get_read_qual_fast(a_art.homo_plus, a_read_2.bpos, a_read_2.seq_ref, a_read_2.seq_read, aln2, qual_2, start_cyc, num_flow_cycles);
                 }
                 else{
-                  qdist.get_read_qual_fast(a_art.homo_minus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual, start_cyc);
-                  ok=qdist.get_read_qual_fast(a_art.homo_minus, a_read_2.bpos, a_read_2.seq_ref, a_read_2.seq_read, aln2, qual_2, start_cyc);
+                  qdist.get_read_qual_fast(a_art.homo_minus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual, start_cyc, num_flow_cycles);
+                  ok=qdist.get_read_qual_fast(a_art.homo_minus, a_read_2.bpos, a_read_2.seq_ref, a_read_2.seq_read, aln2, qual_2, start_cyc, num_flow_cycles);
                 }
 		if(!ok) continue;
 
@@ -245,22 +277,25 @@ int main(int argc, char* argv[]){
                 }
                 FQFILE<<endl;
 
-                ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
-                if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
-                else ALNFILE<<"\t-\n";
-                ALNFILE<<aln[0]<<endl<<aln[1]<<endl;
-
+		if(aln_out){
+		       	ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
+		       	if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
+		       	else ALNFILE<<"\t-\n";
+		       	ALNFILE<<aln[0]<<endl<<aln[1]<<endl;
+		}	
                 //print second read
                 FQFILE2<<"@"<<read_id_2<<endl<<a_read_2.seq_read<<endl<<"+"<<endl;
                 for(size_t k=0; k<qual_2.size(); k++){
                     FQFILE2<<(char)(qual_2[k]+33);
                 }
                 FQFILE2<<endl;
-
-                ALNFILE2<<">"<<id<<"\t"<<read_id_2<<"\t"<<a_read_2.bpos;
-                if(a_read_2.is_plus_strand) ALNFILE2<<"\t+\n";
-                else ALNFILE2<<"\t-\n";
-                ALNFILE2<<aln2[0]<<endl<<aln2[1]<<endl;
+    
+		if(aln_out) {
+		       	ALNFILE2<<">"<<id<<"\t"<<read_id_2<<"\t"<<a_read_2.bpos;
+		       	if(a_read_2.is_plus_strand) ALNFILE2<<"\t+\n";
+		       	else ALNFILE2<<"\t-\n";
+		       	ALNFILE2<<aln2[0]<<endl<<aln2[1]<<endl;
+		}
 
 		if(sam_out){
 		       	sR.qname=srID;
@@ -333,7 +368,7 @@ int main(int argc, char* argv[]){
             }
         }
         FQFILE2.close();
-        ALNFILE2.close();
+       	if(aln_out) { ALNFILE2.close();}
     }
     else{
 	    //single-end reads simulaiton
@@ -363,10 +398,10 @@ int main(int argc, char* argv[]){
 	       	}
                 
                 if(a_read.is_plus_strand){ 
-                  qdist.get_read_qual_fast(a_art.homo_plus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual,start_cyc);
+                  qdist.get_read_qual_fast(a_art.homo_plus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual,start_cyc, num_flow_cycles);
                 }
                 else{
-                  qdist.get_read_qual_fast(a_art.homo_minus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual,start_cyc);
+                  qdist.get_read_qual_fast(a_art.homo_minus, a_read.bpos, a_read.seq_ref, a_read.seq_read, aln, qual,start_cyc, num_flow_cycles);
                 }
 
                 //statistics
@@ -386,10 +421,12 @@ int main(int argc, char* argv[]){
                 }
                 FQFILE<<endl;
 	
-                ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
-                if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
-                else ALNFILE<<"\t-\n";
-                ALNFILE<<aln[0]<<endl<<aln[1]<<endl;
+		if(aln_out){
+		       	ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
+		       	if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
+		       	else ALNFILE<<"\t-\n";
+		       	ALNFILE<<aln[0]<<endl<<aln[1]<<endl;
+		}
 
 		if(sam_out){
 		       	sR.qname=read_id;
@@ -430,7 +467,7 @@ int main(int argc, char* argv[]){
     }
 
     FQFILE.close();
-    ALNFILE.close();
+    if(aln_out){ ALNFILE.close(); }
     STATFILE.close();
 
     if(sam_out){ SAMFILE.close(); }
