@@ -1,22 +1,34 @@
 //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 //ART_454 -- Artificial Read Transcriber 
-//Copyright(c) 2008-2011 Weichun Huang, All Rights Reserved.
+//Copyright(c) 2008-2012 Weichun Huang, All Rights Reserved.
 //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 #include <cmath>
 #include <cstdlib>
 #include <ctime>
 #include "read_profile.h"
 
-void read_profile::default_profile(){
+void read_profile::default_profile(bool is_titanium){
     istringstream  distss;
-    distss.str(QUAL_DIST_1BASE);
-    read_emp_dist(distss,qual_1stbase_dist);
-    distss.clear();
-    distss.str(QUAL_DIST_MC);
-    read_emp_dist(distss,qual_dist_mc, 3);
-    distss.clear();
-    distss.str(READ_LEN_DIST);
-    read_emp_dist(distss,read_len_dist);
+    if(is_titanium){
+	    distss.str(t_QUAL_DIST_1BASE);
+	    read_emp_dist(distss,qual_1stbase_dist);
+	    distss.clear();
+	    distss.str(t_QUAL_DIST_MC);
+	    read_emp_dist(distss,qual_dist_mc, 3);
+	    distss.clear();
+	    distss.str(t_READ_LEN_DIST);
+	    read_emp_dist(distss,read_len_dist);
+    }
+    else{
+	    distss.str(QUAL_DIST_1BASE);
+	    read_emp_dist(distss,qual_1stbase_dist);
+	    distss.clear();
+	    distss.str(QUAL_DIST_MC);
+	    read_emp_dist(distss,qual_dist_mc, 3);
+	    distss.clear();
+	    distss.str(READ_LEN_DIST);
+	    read_emp_dist(distss,read_len_dist);
+    }
     srand ((unsigned int)time(NULL) );
     const int NUM=10;
     double Undercall[10]={0.0000000000, 0.001096668, 0.004562394, 0.01875503, 0.05466385, 0.14088050, 0.26817640, 0.44392523, 0.54782609, 0.69696970};
@@ -25,6 +37,7 @@ void read_profile::default_profile(){
     under_call_error.assign(Undercall,Undercall+NUM);
     over_call_error.assign(Overcall,Overcall+NUM);
     total_error.assign(OverErroRate,OverErroRate+NUM);
+    cyc_base[0]='T'; cyc_base[1]='A'; cyc_base[2]='C'; cyc_base[3]='G';
 }
 
 void read_profile::user_profile(string qual_profile, string mc_qual_profile, string indel_erro_profile, string length_profile){
@@ -39,13 +52,22 @@ void read_profile::user_profile(string qual_profile, string mc_qual_profile, str
 	    total_error[i]=over_call_error[i]+under_call_error[i];
     }
     srand ((unsigned int)time(NULL) );
+    cyc_base[0]='T'; cyc_base[1]='A'; cyc_base[2]='C'; cyc_base[3]='G';
 }
 
 void read_profile::read_indel_error(string error_profile_file){
+	//if file is blank, use the default error profile
     ifstream input(error_profile_file.c_str());
     if(!input){
-        cerr<<"error: cannot open the error profile file "<<error_profile_file<<endl;
-        exit(1); 
+		const int NUM=10;
+	       	double Undercall[10]={0.0000000000, 0.001096668, 0.004562394, 0.01875503, 0.05466385, 0.14088050, 0.26817640, 0.44392523, 0.54782609, 0.69696970};
+	       	double Overcall[]={0.0005165388, 0.000858758, 0.003219707, 0.01025744, 0.01770155, 0.01677149, 0.02264601, 0.01401869, 0.02608696, 0.03030303};
+	       	double OverErroRate[]={0.0005165388, 0.001955426, 0.007782101, 0.02901247, 0.07236540, 0.15765199, 0.29082241, 0.45794393, 0.57391304, 0.72727273};
+	       	under_call_error.assign(Undercall,Undercall+NUM);
+	       	over_call_error.assign(Overcall,Overcall+NUM);
+	       	total_error.assign(OverErroRate,OverErroRate+NUM);
+//	       	cerr<<"Note: "<<error_profile_file<<" does not exist. The default built-in indel_error_profile is used for the simulation"<<endl;
+		return;
     }
     bool is_UC_profile=false;
     bool is_OC_profile=false;
@@ -97,10 +119,20 @@ bool read_profile::get_read_qual_fast(vector<int>& lenVec, unsigned long b_pos,
     seq.reserve(read_seq.size()+10);
     qual.reserve(read_seq.size()+10);
     size_t len=read_seq.length();
-    char aBase;
+    char aBase, orgBase, cycB;
     short polymer_len;
-    for(size_t i=0; i<len;){
-        aBase=read_seq[i];
+    for(size_t i=0, k=0; i<len; k++){
+	    if(k>3){ k=0; cyc++; } 
+	    //stop if max_cycle reached
+	    if(cyc>cycle){
+		    read_seq=read_seq.substr(0,i);
+		    break;
+	    }
+	    cycB=cyc_base[k];
+	    aBase=read_seq[i];
+	    if(cycB!=aBase){ continue; }
+
+	    orgBase=aBase;
         polymer_len=lenVec[b_pos+i];
         i+=polymer_len;
        	//last homolymer (trucated)
@@ -119,7 +151,7 @@ bool read_profile::get_read_qual_fast(vector<int>& lenVec, unsigned long b_pos,
                 if(qvalue<=min_qvalue) qvalue=min_qvalue;
                 polymer_len--;
             }
-            aln[0].append(org_len-polymer_len, aBase);
+            aln[0].append(org_len-polymer_len, orgBase);
             aln[1].append(org_len-polymer_len, aBase);
             seq.append(org_len-polymer_len,aBase);
         }
@@ -181,19 +213,19 @@ bool read_profile::get_read_qual_fast(vector<int>& lenVec, unsigned long b_pos,
         }
 
         if(ins_len>0){
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[0].append(ins_len, '-');
             aln[1].append(polymer_len+ins_len, aBase);
             seq.append(polymer_len+ins_len,aBase);
         }
         else if(del_len>0){
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[1].append(polymer_len-del_len, aBase);
             aln[1].append(del_len, '-');
             seq.append(polymer_len-del_len,aBase);
         }
         else{
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[1].append(polymer_len, aBase);
             seq.append(polymer_len, aBase);
         }
@@ -216,12 +248,6 @@ bool read_profile::get_read_qual_fast(vector<int>& lenVec, unsigned long b_pos,
             qual.push_back(qvalue);
         }
 
-        //stop if max_cycle reached
-        cyc++;
-        if(cyc==cycle){
-            read_seq=read_seq.substr(0,i);
-            break;
-        }
     }
        	return true;
 }
@@ -240,10 +266,19 @@ bool read_profile::get_read_qual(string& read_seq, string& seq, vector<string>&a
     qual.reserve(read_seq.size()+10);
     short cyc=0;
     size_t len=read_seq.length();
-    char aBase;
+    char aBase, orgBase, cycB;
     short polymer_len;
-    for(size_t i=0; i<len;){
-        aBase=read_seq[i];
+    for(size_t i=0, k=0; i<len; k++){
+	    if(k>3){ k=0; cyc++; } 
+	    //stop if max_cycle reached
+	    if(cyc>cycle){
+		    read_seq=read_seq.substr(0,i);
+		    break;
+	    }
+	    cycB=cyc_base[k];
+	    aBase=read_seq[i];
+	    if(cycB!=aBase){ continue; }
+        orgBase=aBase;
         polymer_len=0;
         while(i<len && aBase==read_seq[i]){
             i++;
@@ -261,7 +296,7 @@ bool read_profile::get_read_qual(string& read_seq, string& seq, vector<string>&a
                 if(qvalue<=min_qvalue) qvalue=min_qvalue;
                 polymer_len--;
             }
-            aln[0].append(org_len-polymer_len, aBase);
+            aln[0].append(org_len-polymer_len, orgBase);
             aln[1].append(org_len-polymer_len, aBase);
         }
 
@@ -322,19 +357,19 @@ bool read_profile::get_read_qual(string& read_seq, string& seq, vector<string>&a
         }
 
         if(ins_len>0){
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[0].append(ins_len, '-');
             aln[1].append(polymer_len+ins_len, aBase);
             seq.append(polymer_len+ins_len,aBase);
         }
         else if(del_len>0){
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[1].append(polymer_len-del_len, aBase);
             aln[1].append(del_len, '-');
             seq.append(polymer_len-del_len,aBase);
         }
         else{
-            aln[0].append(polymer_len, aBase);
+            aln[0].append(polymer_len, orgBase);
             aln[1].append(polymer_len, aBase);
             seq.append(polymer_len, aBase);
         }
@@ -357,12 +392,6 @@ bool read_profile::get_read_qual(string& read_seq, string& seq, vector<string>&a
             qual.push_back(qvalue);
         }
 
-        //stop if max_cycle reached
-        cyc++;
-        if(cyc==cycle){
-            read_seq=read_seq.substr(0,i);
-            break;
-        }
     }
 }
 
